/* This file is part of the Cyclone Library.
   Copyright (C) 2001 Greg Morrisett, AT&T

   This library is free software; you can redistribute it and/or it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place, Suite
   330, Boston, MA 02111-1307 USA. */

// This is implements the stack used by exceptions and lexical regions.

#include "runtime_internal.h"

//#define __RUNTIME_STACK_DEBUG
//#define  __RUNTIME_STACK_DEBUG_LOOKUP
//#define __RUNTIME_STACK_DEBUG_GET_EFFECT_FRAME
//  #define __RUNTIME_STACK_DEBUG_THREAD

#ifdef  __RUNTIME_STACK_DEBUG_THREAD
 #define dbgmsg_thread(arg...)  { fprintf(stderr,##arg); fflush(stderr);} 
#else
 #define dbgmsg_thread(arg...)  {} 
#endif

#ifdef  __RUNTIME_STACK_DEBUG_LOOKUP
 #define dbgmsg_lup(arg...)  { fprintf(stderr,##arg); fflush(stderr);} 
#else
 #define dbgmsg_lup(arg...)  {} 
#endif

#ifdef  __RUNTIME_STACK_DEBUG_GET_EFFECT_FRAME
 #define dbgmsg_gef(arg...)  { fprintf(stderr,##arg); fflush(stderr);} 
#else
 #define dbgmsg_gef(arg...)  {} 
#endif

#ifdef  __RUNTIME_STACK_DEBUG 
 #define dbgmsg(arg...)  { fprintf(stderr,##arg); fflush(stderr);} 
#else
 #define dbgmsg(arg...)  {} 
#endif

/************************************************************************/
#define get_current_effect_frame() LPAREN {\
  struct _EffectFrame *ret = (struct _EffectFrame *) effect_frame();\
  dbgmsg_gef("\n[TID=%p] GET  effect frame: %p \n",pthread_self(),ret);\
   ret;\
  } RPAREN

#define get_current_frame() (abstract_frame())
#define set_current_frame(x) (set_abstract_frame(x))

////////////////////////////////////////////////////////////////////////
struct _RuntimeStack * _top_frame() {
  return get_current_frame();
}
void _push_frame(struct _RuntimeStack *frame) {
  struct _ThreadKey *t = tcb();
  frame->next = t->stack._current_frame;
  t->stack._current_frame = frame;
}

// set _current_frame to its n+1'th tail (i.e. pop n+1 frames)
// Invariant: result is non-null
void _npop_frame(unsigned int n) {
  struct _ThreadKey *t = tcb();
  struct _RuntimeStack *current_frame = t->stack._current_frame;
  unsigned int i;

  for(i = n; i <= n; i--) {
    if(current_frame == NULL) {
      errquit("internal error: empty frame stack\n");
    } 
  	dbgmsg("\nPOPING frame : %p code = %p" , current_frame, current_frame->cleanup);
	    if (current_frame->cleanup != NULL)
   	   current_frame->cleanup(current_frame);
    current_frame = current_frame->next;
  }
  t->stack._current_frame = current_frame;
}
/***********************************************************************/
#define tr(x) ((struct _XTreeHandle *)x)
static inline void * __lookup_effect( unsigned int idx, 
								struct _EffectFrame *curr_frame);

#ifdef HAVE_THREADS
static volatile void *once_thread_init = 0;
tlocal_key_t _thread_key;
#endif

void _init_thread_context( struct _ThreadKey *key){
	  
	if( key == NULL )  errquit("_init_thread_context");

	#ifdef HAVE_THREADS
	  int status;
	  if( AO_compare_and_swap_release((ao_t *)&once_thread_init,
					  									(ao_t )0,(ao_t )1) != 0) {
			if((status = create_tlocal_key(&_thread_key,NULL)))
   			 errquit("Thread local storage key creation failed (%d)", 
						 	status);
	  }
	  put_tlocal(_thread_key, key);
	#endif

	 // if( key == &_main_key ) // I need to initialize main thread
		  memset(key,0,sizeof(struct _ThreadKey));	 
		  key-> h =  create_hashtable(25);
	  	  key->tid = (void *) key; // unique !
	  

	  /*	 pthread_attr_t attr;
		  void *base=NULL;
		  size_t size=0;*/
				// * getting a stack address and size * /
			 /* if(pthread_attr_init(&attr) != 0)
				  errquit("pthread_attr_init");*/
			//  if(pthread_attr_getstack (&attr, &base, &size) != 0)
			//	  errquit("pthread_attr_getstack");
			//
			/*  if( pthread_attr_getstacksize(&attr, &size) != 0 )
				  errquit("pthread_attr_getstacksize");
			  if( pthread_attr_getstackaddr(&attr, &base) != 0 )
					  errquit("pthread_attr_getstackaddr");
			  if( base == NULL ) base = key;*/
			 /* size_t guard = 0;
			  pthread_attr_getguardsize(&attr,&guard);
			  	printf("\nGet stack guard size: %d\n",guard);fflush(stdout);
			  	// 1MB = 0x400 , 
			  if( size > 0x800 ){
				  size = 0x800;
				  pthread_attr_setstacksize(&attr,size);
			  }*/
	//		 printf("\n!! base = %p last=%p size=%d\n",base,key+size,size/1024);
			  /*if( pthread_attr_destroy(&attr) != 0 )
				  errquit("pthread_attr_destroy");*/
}

#ifdef HAVE_THREADS
static inline void _fini_thread_context(){
	struct hashtable *h = tcb()->h;
	if( h != NULL) hashtable_destroy(h);
}
////////////////////////////////////////////////////////
#define ALIGN_SZ(x)  (((unsigned long)(x) & ~0x7) + 0x8)
#ifndef PAGE_SIZE
#define PAGE_SIZE 4096
#endif
#define ALIGN_PG(p)   ((char *)(((long long) p + PAGE_SIZE-1) & (long long)~(PAGE_SIZE-1)))
#ifndef PROT_READ
#define PROT_READ 1
#endif

#define ts(x) ((thread_stack_t *)x)
#define BACKUP_STK_SZ  PAGE_SIZE
typedef struct ThreadStack {
	struct _ThreadKey key;
	void  *real_base; // useful for GC_free or free
	struct _EffectFrame fst; 
	struct _EffectFrame snd;
	void (*thread)(void *);
	void *arg;
	char __backup__stack[BACKUP_STK_SZ]; // suitable for catching sigsegv
	char  free_space[];
} thread_stack_t;

void *_thread_internal2( void *arg){
	  if( arg == NULL ||   &(ts(arg)->key) == NULL)
		  errquit("_thread_internal2 arg=%p arg->key=%p\n",
				    arg, &(ts(arg)->key));
	  put_tlocal(_thread_key, (void *) &(ts(arg)->key));
	if(stackoverflow_install_handler(&_exn_stackoverflow_handler,   
												ts(arg)->__backup__stack, 
												BACKUP_STK_SZ) < 0){  
					errquit("Could not install stack handler");
	}
	sigsegv_install_handler(&_exn_segv_handler);

  if( _set_top_handler() == NULL ) {
	  // set second at the top of the stack. 
	  // the remaining frames are already in the linked list
	  ts(arg)->key.stack._current_frame = &ts(arg)->snd.s;
	  (ts(arg)->thread)(ts(arg)->arg);
  }

	 
	/* Mark the page W/R. */
    if (ts(arg)->real_base == arg && 
			 mprotect((void *)ts(arg)->key.stack_ceil,1024,3))
			 errquit("__thread_internal2: mprotect failed.");

  _fini_thread_context();
  AO_barrier_add(&all_done,-1);
  pthread_exit(0);							  
  return NULL;

}

void _effect_frame_cleanup(struct _RuntimeStack *r);
int mprotect(const void *addr, size_t len, int prot);

thread_stack_t *__init_new_thread_stack( const unsigned int desired_stack_size,
													  struct _EffectFrame *curr_frame,
													  const struct _EffectFrameCaps *caps,
													  const unsigned int max_caps,
													 		  void (*thread)(void*),
													  		  void *arg, 
													  const unsigned char *offsets,
													  const unsigned int max_formal,
													  const unsigned int sizeof_arg
													){
	static const unsigned int MB = 1048576;
	static unsigned int PADDING = 16; // it should be at least 16 or we'll be
												 // in trouble see ALIGN_SZ
	const unsigned long default_allocation =  
		ALIGN_SZ(  ( sizeof(struct _XTreeHandle) 
					   + sizeof(void *)) * max_caps
					   + sizeof(void *) * max_formal 
					//it should be sizeof(char) but we 
					// care about being aligned
					+ sizeof(thread_stack_t) + sizeof_arg + PADDING); 
	thread_stack_t *stack = NULL;

	const unsigned int min_allocation = SIGSTKSZ + PAGE_SIZE;
	unsigned int stack_estimation = 0;

	if( min_allocation > desired_stack_size )
		stack_estimation = default_allocation + 
								 min_allocation + PAGE_SIZE ;
	else
		stack_estimation = default_allocation + desired_stack_size + PAGE_SIZE;

	if( stack_estimation < SIGSTKSZ )
		stack_estimation = default_allocation + SIGSTKSZ + PAGE_SIZE;

	if( ALIGN_PG(stack_estimation) != stack_estimation )
		stack_estimation = ALIGN_PG(stack_estimation) + PAGE_SIZE;

//	printf("\nStack size : %d in pages: %d allocation: %d\n",stack_size,stack_size/PAGE_SIZE,default_allocation); fflush(stdout);
//   stack = 	(thread_stack_t *) GC_malloc_uncollectable(stack_size*sizeof(char));
   stack = 	(thread_stack_t *) GC_malloc(stack_estimation*sizeof(char));
	if( stack == NULL ) goto error;

	if( stack != (char *) ALIGN_PG(stack)   ){
			//	errquit("__init_new_thread: unaligned stack");
		void *base = (void *) stack;
		void *aligned = ALIGN_PG(stack) + PAGE_SIZE;
		stack = (thread_stack_t *) aligned;
		stack->real_base = base;
		stack_estimation -= PAGE_SIZE;
	} else stack->real_base = stack;
   
	const unsigned int stack_size = stack_estimation;

	char *guard = (char *)stack + stack_size - PAGE_SIZE;
	if( guard != (char *)ALIGN_PG(guard) )
				errquit("__init_new_thread: unaligned guard");

/*	if( stack + default_allocation != (char *) 
			ALIGN_PG(stack + default_allocation)   )
				errquit("__init_new_thread: unaligned start addr");*/

  	 /* Mark the page read-only. */
    if (stack->real_base == stack && mprotect(guard,1024,1))
			 errquit("__init_new_thread: mprotect failed.");
 
	// if NULL throw bad_alloc_exn
	char *next_addr = (char*)&(stack->free_space[0]);	
	struct _EffectFrame *fst = &(stack->fst);
	struct _EffectFrame *snd = &(stack->snd);
	struct _XTreeHandle *roots,*h,*iter;
	int idx,i,j;

    memset((void *)stack,0,sizeof(thread_stack_t));//initialize struct
	 stack->key.tid = (void *) stack; // unique !  and != NULL
	 stack->key.stack_ceil = (void *) guard;
	 stack->key.h =  create_hashtable(25); // heap-allocated
	 stack->key.stack_base = (void *) stack + default_allocation;

	 stack->thread = thread;
		
	 // IMPORTANT: we are **not** setting up the exception handler
	 // nor stack->stack._current_frame as the exception handler needs
	 //  to be inserted first and this must be done in the new thread !!!!
	 fst->s.tag = 2;
	 snd->s.tag = 2;
	 snd->max_formal = max_formal;
	 fst->s.cleanup = (void(*)(struct _RuntimeStack*)) _effect_frame_cleanup;
	 snd->s.cleanup = (void(*)(struct _RuntimeStack*)) _effect_frame_cleanup;
	 snd->s.next =  &fst->s;
	 // the exception handle should be at the top of the stack
	 fst->s.next = &stack->key.exn.top_handler.s;
	 // we can set-up the current effect frame though
	 stack->key.stack._current_effect_frame = snd;
	 
	 	 
	// allocate handles for fst
   fst->inner_handles =  (void **) next_addr;
	next_addr += sizeof(void *) *	max_caps;
	//allocate handles for snd
   snd->formal_handles =  (char *) next_addr;
	next_addr += sizeof(void *) *	max_formal;
	next_addr = (void *)ALIGN_SZ(next_addr);
	// allocate struct for roots
	roots = (struct _XTreeHandle *) next_addr;
	next_addr += sizeof(struct _XTreeHandle)* max_caps;
	next_addr = (void *) ALIGN_SZ(next_addr);

	stack->arg = next_addr;
	memcpy(next_addr,arg,sizeof_arg); //copy argument list
	next_addr += sizeof_arg;
	next_addr = (void *) ALIGN_SZ(next_addr);


	if( (long long) stack->key.stack_base & (long long) 0x7 != 0 )
	  errquit("_init_new_thread_stack: unaligned base.");

	if( (void *) next_addr > stack->key.stack_base)
	  errquit("_init_new_thread_stack: next_addr (%d) > base (%d)",next_addr,
			  	 stack->key.stack_base);
	
	// INVARIANT: max_caps may be less than max_formal as we may pass aliases.
	// 			  Under no circumstances can it be greater than max_formal
	if( max_formal < max_caps)
			errquit("_init_new_thread_stack: max_caps (%d)  >  max_formal (%d)",
						max_caps,max_formal);

	// caps[i].tidx -> index at the formal_handles array 
	// curr_frame->formal_handles[caps[i].tidx] -> 
   // caps[i].idx  -> 
	//	for( i = 0 ; i < max_caps ; i++ ){
	//		printf("\ncaps[%d] = %d\n",i,caps[i].tidx);
	//}

	// make temporary space for the spawning thread's handle table
	struct _XTreeHandle **local = (struct _XTreeHandle **) 
													alloca(sizeof(struct _XTreeHandle*) * max_caps);

	// initialize snd->formal_handles
	memcpy(snd->formal_handles,offsets,max_formal); // fast copy
	memset(roots,0,max_caps*sizeof(struct _XTreeHandle)); // initialize roots

	//curr_frame = (struct _EffectFrame *) curr_frame->s.next;

	// First round: do the hashtable insertions and store remote
	// XTreeHandles temporarily to this thread's space
	for( i = 0 ; i < max_caps ; i++ ){
			idx = caps[i].idx; 
			h = (struct _XTreeHandle *) __lookup_effect(idx,curr_frame);
			local[i] = h;
			dbgmsg_thread("local[%d] = %p (%d) --> parent = %p\n",
							  i,h,idx,h->parent_key);
			fst->inner_handles[i] = &roots[i];
			roots[i].h = h->h; //this operation must ALWAYS 
										  // take place before hastable_insert 
										  // 	as "h" is the key
			hashtable_insert(&stack->key.h,&roots[i]); // insert to hashtable
	}
	dbgmsg_thread("\n");
		// Second round: build tree
		// assume ordered  array so that [childen ... parents]

		// IMPORTANT NOTE: children of the original tree may be in different
		// 					 order than the children of the new thread ...
		// 					 does this matter when it comes to locking trees?
		// 					 Yes it does! It may cause a deadlock if the tree
		// 					 is locked bottom up !
		//
		// 					 We don't care here as locking is performed Top-Down
		for( i = 0 ; i < max_caps ; i++ ){
			h = local[i];
			dbgmsg_thread("local[%d] = %p --->  %p\n",i,h,&roots[i]);
		   if( h->parent_key == NULL ) break; // ignore parents
			h = h->parent_key;
			for( j = max_caps -1 ; j > 0 && j > i  ; j-- )
				if( local[j] == h ) break;
			if( j < 0  ) 
				errquit("_thread internal: could not find parent");
			roots[i].parent_key = &roots[j];
			// register to parent
			if(roots[j].sub_keys != NULL )
			roots[i].next_key =  roots[j].sub_keys;
			roots[j].sub_keys = &roots[i]; 
		}
		/*for( i = 0  ; i < max_caps ; i++ ){
			print_xtree(&roots[i] ,0);
		printf("\n++++++++++++++++++++++++++++++++++++++++++++++++\n");
		}*/
		//exit(0);
		// Setup capabilities
		//start off from the child regions
		// can't start from root, unsafe. unless deallocated children are
		// deleted from the candidate list
		// printf("\n\n\n\n\n THREAD STAGE 3 \n\n\n\n\n"); fflush(stdout);
		AO_lock_val_t self = (AO_lock_val_t) stack->key.tid;
		for( i = 0  ; i < max_caps ; i++ ){
			//idx = caps[i].idx; 
			h = local[i]; 
			// increment global count before releasing other counts
			ao_t rg_cnt=AO_fetch_and_add_release(&(h->h->caps.rcnt),(ao_t)1);
			int old_rg = h->caps.rcnt - caps[i].rg; // any left?
			// this thread
			if( caps[i].lk > 0 ){  //transfer lock 
				__xlsteal(&roots[i],caps[i].lk,0,self);
			}
			__xinc(&roots[i],caps[i].rg); // add counts to this thread
			//other thread
			if( caps[i].lk > 0 ){
				 __xldec(h,caps[i].lk,0);
			}
			__xdec(h,caps[i].rg); 
			//  the next step must be performed after
			if( rg_cnt == 1 && old_rg == 0 ) // if thread-local region set singlethread key
		 		  AO_store_release((ao_t *)&(roots[i].h->single_thread_key),
						  				 (ao_t) &roots[i]);
			else
				AO_store_release((ao_t *)&(roots[i].h->single_thread_key),0);
			//fst->inner_handles[caps[i].tidx] = &roots[i];
			//print_xtree_shallow(fst->inner_handles[caps[i].tidx] ,0);
			//printf("\n------@@@@@ THREAD internal @@@\n");
		}
		/*
		printf("\n------>>>#############>>> THREAD internal\n");
		for( i = 0  ; i < max_caps ; i++ ){
			print_xtree(&roots[i] ,0);
		printf("\n++++++++++++++++++++++++++++++++++++++++++++++++\n");
		}
		printf("\n------>>>#########>>> END THREAD internal\n");
		exit(0);*/
		return stack;
	  error:
			if( stack != NULL && stack->key.h != NULL ){
			  		hashtable_destroy(stack->key.h);
					stack->key.h = NULL;
			}
			if( stack != NULL ) GC_free(stack->real_base);	 
			// throw exn ?
			return NULL;
	

}

/*void _bg(){
	printf("\nHERE !\n");
//	exit(0);
}*/
void _spawn_with_effect( const struct _EffectFrameCaps *caps,
								 const unsigned int max_caps,
								 		 void (*f)(void *) , 
								 		 void *targ,
								 const unsigned char *offsets,
								 const unsigned int   max_formal,
								 const unsigned int	 sizeof_targ,
								 const unsigned int	 desired_stack_size

							  ){
	static const unsigned int MB = 1048576;
	pthread_attr_t attr;
	pthread_t id;
	int ret;
	struct _EffectFrame *curr_frame = get_current_effect_frame();

		thread_stack_t *stack = 
			__init_new_thread_stack(desired_stack_size,curr_frame
											,caps,max_caps,f,targ,offsets,max_formal,
											sizeof_targ);

		 if( stack == NULL )
				  errquit("_spawn_with_effect: could not allocate stack");

		// * getting a stack address and size * /
		 if(pthread_attr_init(&attr) != 0)
				  errquit("pthread_attr_init");

		 //IMPORTANT: reclaim memory		 
       if (pthread_attr_setdetachstate(&attr, 
								PTHREAD_CREATE_DETACHED) != 0)
                  errquit("pthread_attr_setdetachstate");
		 AO_barrier_add(&all_done,1);
 		 if( (ret=pthread_attr_setstack(&attr,stack->key.stack_base, 
					 						 stack->key.stack_ceil -
				 							 stack->key.stack_base	 )) != 0 )
               errquit("pthread_attr_setstack: %d",ret);
	
		 if( GC_pthread_create(&id,&attr,_thread_internal2,(void *)stack) != 0 )
				errquit("internal error: _spawn_with_effect could not create thread\n");
	
		 if( pthread_attr_destroy(&attr) != 0 )
			  errquit("pthread_attr_destroy");
  //arg.done = id;
		//AO_wait_lock(&arg.done);
		//while(arg.done == 0); // spin-lock
		/*for( i = 0 ; i < max_caps ; i++ ){
			struct _XTreeHandle *h = tr(__lookup_effect(caps[i].idx,fr));
			if( h->parent_key == NULL ){
					printf("\n@@@ PARENT THREAD AFTER SPAWN");
					_print_xtree(h,0);
					fflush(stdout);
			}
		}*/
	//fr->s.tag = 2;
   //fr->s.next = (struct _RuntimeStack *) curr_frame;
	//fr->s.cleanup = NULL;	
	//realthread_t arg = {f,targ,fr,caps,max_caps,0};
	/*	for( i = 0 ; i < max_caps ; i++ ){
			struct _XTreeHandle *h = tr(__lookup_effect(caps[i].idx,fr));
			if( h->parent_key == NULL ){
					printf("\n@@@ PARENT THREAD BEFORE SPAWN");
					_print_xtree(h,0);
					fflush(stdout);
			}
		}
		sleep(1);*/
		/*for( i  =0 ; i < max_caps ; i++ ){
			struct _XTreeHandle *h = (struct _XTreeHandle *) 
								 __lookup_effect(caps[i].idx,curr_frame);
			printf("\nidx = %d,  ptr=%p\n",caps[i].idx,		h);
			print_xtree_shallow(h,0);
		}*/
	   /*printf("\n.................$$$$$$......................\n");
		for( i  =0 ; i < max_caps ; i++ )
		print_xtree_shallow(stack->fst.inner_handles[i],0);

	   printf("\n.................$$$$$$......................\n");

	   printf("\n.................$$$$$$......................\n");

	   printf("\n.................$$$$$$......................\n");
		for( i  =0 ; i < max_caps ; i++ ){
			struct _XTreeHandle *h = (struct _XTreeHandle *) 
								 __lookup_effect(caps[i].idx,curr_frame);
			printf("\nidx = %d,  ptr=%p\n",caps[i].idx,		h);
			print_xtree_shallow(h,0);
		}*/

		//GC_free(stack);
		  // 1MB = 0x400 , 
		/* if( size > MB ){
				  size = MB;
				 if( (size=pthread_attr_setstacksize(&attr,size)) != 0)
				  errquit("pthread_attr_getstacksize (%d)",size);
		 }*/

	 //	_bg();
//	if( pthread_create(&id,&attr,_thread_internal2,&arg) == 0 ){
}
#else
/*void _spawn( void (*f)(void *) , void *targ ){
	f(targ);
}*/
#endif


/* only inner  function handles can be registered */
void _register_effect(unsigned int idx, void *handle ){
  struct _EffectFrame *curr_frame =  get_current_effect_frame();
  unsigned int max=  curr_frame->max_formal ;
  if( idx < max ){
	    errquit("internal error: _register_effect.\n");
  }
  dbgmsg("\n[FRAME = %p] REGISTERING addr = %p  idx= %d max = %d handle = %p\n",curr_frame,curr_frame->inner_handles+(idx-max),idx,max,handle); 
  (curr_frame->inner_handles)[idx-max] = handle;
  dbgmsg("\n[FRAME = %p] REGISTERING addr = %p  idx= %d max = %d handle = %p stored = %p\n",curr_frame,curr_frame->inner_handles+(idx-max),idx,max,handle, (curr_frame->inner_handles)[(idx-max)]); 
  /*
  struct _InnerEffectFrame inner;	
  int i;
  if (curr_frame == NULL || (curr_frame->inner) == NULL ) 
	    errquit("internal error: _register_effect.");
  inner = *(curr_frame->inner);
  for( i = 0 ; i < inner.curr && 
		  			strcmp(inner.effects[i].name,rgn_name) != 0 ; i++);
  if( i == inner.curr && i == inner.max ){
	    errquit("internal error: _register_effect.");
  }
  else if( i == inner.curr ) (curr_frame->inner->curr)++;
  inner.effects[i].name = rgn_name;
  inner.effects[i].key = handle;*/
}

  /*
static inline void * __lookup_effect( unsigned int idx, 
								struct _EffectFrame *curr_frame){
//  struct rgn_map_pair *effects;
  for( i = 0,effects = curr_frame->inner->effects, 
		  max = curr_frame->inner->curr;
		  i <  max && strcmp( effects[i].name,rgn_name) != 0 ; i++);
  if( i < max  ) return effects[i].key;

  for( i = 0,effects = curr_frame->effects, max = curr_frame->max ;
		  i <  max && strcmp( effects[i].name,rgn_name) != 0 ; i++);
  if( i < max  ) return effects[i].key;
   errquit("internal error: _lookup_effect.");
}*/
#define eframe(x) ((struct _EffectFrame *)x)

 void * __lookup_effect( unsigned int idx, 
								struct _EffectFrame *curr_frame){
  unsigned int max;
  struct _RuntimeStack *r;
	  for( r  = (struct _RuntimeStack *) curr_frame; 
			 r != NULL ; r = r->next ){
     if( r->tag != 2 ) continue; //skip
	  max = eframe(r)->max_formal;
	  dbgmsg_lup("\nLooking up idx = %d when max = %d, formal = %p ,inner=%p\n",
		   idx,max,
		  	eframe(r)->formal_handles, eframe(r)->inner_handles);
	  if( idx  >= max ){
		  	  dbgmsg_lup("\nLookup returns : %p at frame %p\n",
					  	eframe(r)->inner_handles[idx-max],
						eframe(r));
		 	 return eframe(r)->inner_handles[idx-max];
	  }
	  else   idx = eframe(r)->formal_handles[idx];
  }
  errquit("internal error: __lookup_effect. thread=%p", pthread_self());
  /*return ( idx < max )?curr_frame->formal_handles[idx]:
							  curr_frame->inner_handles[idx-max];*/
 }

void * _lookup_effect( unsigned int idx){
 // struct _EffectFrame *curr_frame = get_current_effect_frame();
  return  __lookup_effect(idx,get_current_effect_frame());
}

void * _lookup_first_effect(){
//  struct _EffectFrame *curr_frame =  get_current_effect_frame();
  //dbgmsg("\n [FIRST EFFECT]  frame= %p addr = %p handle= %p\n",curr_frame,				curr_frame->formal_handles,(curr_frame->formal_handles)[0]);
//  return (curr_frame->formal_handles)[0];
	return __lookup_effect(0,get_current_effect_frame());
}

void _peek_effect( void **inner_handles ){
  struct _EffectFrame *curr_frame =  get_current_effect_frame();
  if (curr_frame == NULL) 
    errquit("internal error: _peek_effect.");
  //save this for future reference
  dbgmsg("\nPEEK [FRAME = %p] [inner = %p ]  [new inner = %p]\n",
		  	curr_frame, curr_frame->inner_handles, inner_handles);
  curr_frame->inner_handles = inner_handles;
}

void _effect_frame_cleanup(struct _RuntimeStack *r){
 //  unsigned int sz = eframe(r)->max_update,i;
//	struct _EffectFrameCaps *uh = eframe(r)->update_handles;

	//find previous frame
   dbgmsg("\nCLEANUP ENTER\n"); 
	for(r = r->next ; r != NULL && r->tag != 2 ; r = r->next );
/*
	for( i = 0 ; i < sz ; i++ ){
		printf("\n[AUTO REMOVE] handle %p  rg=%d lk=%d\n",
		__lookup_effect(uh[i].idx,eframe(r)),uh[i].rg,uh[i].lk);
	}
	*/
	set_effect_frame((struct _EffectFrame *)r);
   dbgmsg("\nCLEANUP LEAVE : %p\n",r); 
}



#define trgn(x)   ((struct _XTreeHandle *) x)
/* the length of fresh_names should be equal to */
void _push_effect( struct _EffectFrame *r,
						 const struct _EffectFrameCaps *update_handles,
						 const unsigned int max_update  ){
/*
  struct rgn_map_pair *effects;
  int i,max;
  void *key;*/ 
   struct _ThreadKey *t = tcb();	
	struct _EffectFrame *curr_frame =  t->stack._current_effect_frame;
	int i,new_max,max,idx_tmp;
	void **new_formal_handles,**addr;
	struct _XTreeHandle **xh;
	/* Step 1: Remove regions no longer needed  */ 
   if( max_update > 0 ){
	  xh = (struct _XTreeHandle **)
					 alloca( sizeof(struct _XTreeHandle *) *
				  				max_update);
   	/* Step 1.1: Find handles that should be removed */
	  for( i = 0 ; i < max_update ; i++ ){
	 	 xh[i] = tr(__lookup_effect(update_handles[i].idx,curr_frame));
		 if( xh[i] == NULL ){
			 errquit("_push_effect : NULL handle at index %d max_update=%d\n",
					 	update_handles[i].idx,max_update);
		 }
	  }

		/* Step 1.2: Remove those handles */
		__xDecBoth(xh,max_update,update_handles);		
   }

   /* Step 2: initialize data on the new frame */
  	r->s.tag = 2;
   r->s.cleanup =  (void(*)(struct _RuntimeStack*)) _effect_frame_cleanup;
	r->s.next = NULL; 
   //r->inner_handles = NULL;
	dbgmsg("\nPUSHING FRAME = %p ",r);
	//for( i = 0,max=r->max_formal ; i < max ; i++ ){
	//	dbgmsg("f[%d] => %d\t",i,(int) r->formal_handles[i]);
	//}
	//dbgmsg("\n");
//  for( i = 0,effects = r->effects, max = r->max ; i <  max  ; i++){
//		effects[i].key = __lookup_effect(effects[i].name,curr_frame);
 //}
 //
  /* copy the required keys into the new frame*/
	/*
  if( curr_frame != NULL ){
		max = curr_frame->max_formal;
		new_formal_handles = r->formal_handles;
		new_max = r->max_formal;
//		dbgmsg("\nnew_max = %d %d || oldmax= %d\n" , new_max,count++,max); 
		for( i = 0 ; i < new_max ; i++ ){
			idx_tmp = (unsigned char) new_formal_handles[i];
			addr = ( idx_tmp < max )?
							  (curr_frame->formal_handles)+idx_tmp:
						     (curr_frame->inner_handles)+(idx_tmp-max);	
			new_formal_handles[i] = *addr;
		dbgmsg("\n[NEW FORMAL HANDLES] REGISTERING new_addr = %p idx= %d  $$$(old_frame = %p || addr = %p) => oidx= %d oldmax=%d  handle = %p $$\n",new_formal_handles,i,curr_frame,addr,idx_tmp,max,*addr); 
		}
  } */
  t->stack._current_effect_frame = r ;
  r->s.next = t->stack._current_frame;
  t->stack._current_frame = &r->s;
  dbgmsg("\nPUSH EFFECT OK\n");
}

void _pop_effect(){
  struct _RuntimeStack *curr_frame = _top_frame();
 	if (curr_frame == NULL || curr_frame->tag != 2) {
    errquit("internal error: _pop_effect (tag = %d cleanup = %p frame=%X). thread id = %p\n",
			((curr_frame!=(void *)NULL)?curr_frame->tag:0),
		  	((curr_frame!=(void *)NULL)?(void *)curr_frame->cleanup:(void *)NULL),curr_frame,
			pthread_self());
  }
  _npop_frame(0);
}

///////////////////////////////////////////////////////////////////
//
////#ifdef HAVE_THREADS
//static tlocal_key_t _current_frame_key;
//static tlocal_key_t _current_effect_frame_key;
//#else
//static struct _EffectFrame *_current_effect_frame = NULL;
//static struct _RuntimeStack *_current_frame = NULL;
//#endif
/*************************************************************************/
/*
#define MAX_MAP_SZ ((unsigned int) 256)

typedef struct {
	const char *name;
	void *key;
} rgn_map_key_t;

typedef struct {
	rgn_map_key_t elts[MAX_MAP_SZ];
	unsigned char curr_pos[MAX_MAP_SZ]; // may have to change latter
	unsigned int  idx;
} rgn_map_t;


#ifdef HAVE_THREADS
rgn_map_t *init_map(){ 
	rgn_map_t *map = NULL;
  	if((status = create_tlocal_key(&_rgn_map_key, NULL)))
    errquit("Thread local storage key creation failed (%d)", status);
	map =  (rgn_map_t * ) malloc(sizeof(rgn_map_t) * 1);
	if( map == NULL ) 
    errquit("Malloc failed (init_map)");
   put_tlocal(_rgn_map_key, map);
	map->idx = 0;
	map->curr_pos[0] = 0;
}

void push_map(){
}

void pop_map(){
}
void update_map( const char *name, void *key ){
  map = (rgn_map_t *) get_tlocal(_current_frame_key); 
  
}

void *get_map( const char *name ){
  return (struct _RuntimeStack *)get_tlocal(_current_frame_key); 
}
#endif
*/
//#ifdef HAVE_THREADS
//static int once_thread_init_stack = 0;
//#endif
/*
 * void set_current_effect_frame(struct _EffectFrame *frame); 

 * void _init_stack() {
#ifndef HAVE_THREADS
   _current_frame = NULL;
	_current_effect_frame = NULL;
#endif
}*/

